﻿using ProjectOpticsStore.Entities.Enums;
using ProjectOpticsStore.Entities;
using ProjectOpticsStore.Repositories;
using DocumentFormat.OpenXml.Drawing.Charts;

namespace ProjectOpticsStore.Forms;

public partial class FormOrder : Form
{
    private readonly IOrderRepository _orderRepository;
    private readonly ICustomerRepository _customerRepository;
    private readonly IProductRepository _productRepository;


    public FormOrder(IOrderRepository orderRepository, ICustomerRepository customerRepository, IProductRepository productRepository)
    {
        InitializeComponent();
        _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
        _customerRepository = customerRepository ?? throw new ArgumentNullException(nameof(customerRepository));
        _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));

        // Инициализация ComboBox для магазинов
        comboBoxCustomer.DataSource = _customerRepository.ReadCustomers().ToList();
        comboBoxCustomer.DisplayMember = "FullName";  // Отображаем фио заказчиков
        comboBoxCustomer.ValueMember = "Id";        // Значение это идентификатор заказчика
        comboBoxCustomer.DropDownStyle = ComboBoxStyle.DropDownList;

        // Настройка ComboBox для выбора статуса заказа
        comboBoxStatus.DataSource = Enum.GetValues(typeof(OrderStatusEnum));
        comboBoxStatus.DropDownStyle = ComboBoxStyle.DropDownList;

        dataGridViewProducts.AllowUserToAddRows = true;
        dataGridViewProducts.AllowUserToDeleteRows = true;

        // Настройка таблицы товаров
        InitializeDataGridViewProducts();

        // Настройка DateTimePicker для даты заказа
        dateTimePickerOrderDate.Value = DateTime.Now;
        dateTimePickerOrderDate.Enabled = false;
    }

    private void InitializeDataGridViewProducts()
    {
        // Отключаем авто-генерацию столбцов
        dataGridViewProducts.AutoGenerateColumns = false;

        // Устанавливаем режим авторазмера столбцов
        dataGridViewProducts.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;

        // Столбец "Товар"
        var productColumn = new DataGridViewComboBoxColumn
        {
            Name = "ColumnProduct",
            HeaderText = "Товары",
            DataSource = _productRepository.ReadProducts()
                .Select(p => new { p.Id, p.Type }) // Пример данных: Id и Type
                .ToList(),
            DisplayMember = "Type",
            ValueMember = "Id",
            FlatStyle = FlatStyle.Flat
        };
        dataGridViewProducts.Columns.Add(productColumn);

        // Столбец "Количество"
        var quantityColumn = new DataGridViewTextBoxColumn
        {
            Name = "ColumnQuantity",
            HeaderText = "Количество",
            ValueType = typeof(int)
        };
        dataGridViewProducts.Columns.Add(quantityColumn);

        // Столбец "Тип"
        var typeColumn = new DataGridViewTextBoxColumn
        {
            Name = "ColumnType",
            ValueType = typeof(int),
            Visible = false  // Скрываем столбец
        };
        dataGridViewProducts.Columns.Add(typeColumn);

        productColumn.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        quantityColumn.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
    }


    private void ButtonSave_Click(object sender, EventArgs e)
    {
        try
        {
            if (string.IsNullOrWhiteSpace(comboBoxCustomer.Text) || comboBoxStatus.SelectedItem == null || dataGridViewProducts.RowCount < 1)
            {
                throw new Exception("Имеются незаполненные поля");
            }

            var customerId = (int)comboBoxCustomer.SelectedValue!;
            var status = (OrderStatusEnum)comboBoxStatus.SelectedItem!;
            var orderProducts = CreateOrderProductsFromDataGrid();

            var order = Order.CreateOperation(0, customerId, orderProducts, status);
            _orderRepository.CreateOrder(order);

            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonCancel_Click(object sender, EventArgs e)
    {
        Close();
    }

    private List<Order_Product> CreateOrderProductsFromDataGrid()
    {
        var list = new List<Order_Product>();

        foreach (DataGridViewRow row in dataGridViewProducts.Rows)
        {
            // Пропускаем строку, если не указаны значения для продукта или количества
            if (row.Cells["ColumnProduct"].Value == null || row.Cells["ColumnQuantity"].Value == null)
                continue;

            var productId = Convert.ToInt32(row.Cells["ColumnProduct"].Value);
            var quantity = Convert.ToInt32(row.Cells["ColumnQuantity"].Value);
            // Какой же говнокод
            var typeValue = row.Cells["ColumnType"].Value?.ToString();
            ProductTypeEnum type = ProductTypeEnum.Glasses;  

            list.Add(Order_Product.CreateElement(0, productId, type, quantity));
        }

        return list.GroupBy(
            x => new { x.ProductId, x.Type }, // Группируем по ProductId и Type
            x => x.Quantity,
            (key, quantity) => Order_Product.CreateElement(0, key.ProductId, key.Type, quantity.Sum())
        ).ToList();
    }
}