﻿using ProjectOpticsStore.Entities.Enums;
using ProjectOpticsStore.Repositories;

namespace ProjectOpticsStore.Forms;

public partial class FormProduct : Form
{
    private readonly IProductRepository _productRepository;
    private readonly IStoreRepository _storeRepository;
    private int? _productId;

    public int Id
    {
        set
        {
            try
            {
                // Загрузка данных продукта
                var product = _productRepository.ReadProductById(value);
                if (product == null)
                {
                    throw new InvalidDataException(nameof(product));
                }

                // Заполнение значений CheckedListBox
                foreach (ProductTypeEnum elem in Enum.GetValues(typeof(ProductTypeEnum)))
                {
                    if ((elem & product.Type) != 0)
                    {
                        checkedListBoxProductType.SetItemChecked(
                            checkedListBoxProductType.Items.IndexOf(elem), true);
                    }
                }

                // Заполнение текстовых полей
                textBoxPower.Text = product.Power.ToString();
                textBoxPrice.Text = product.Price.ToString();
                comboBoxStore.SelectedValue = product.Store;
                textBoxThickness.Text = product.Thickness.ToString("F2");
                textBoxDisease.Text = product.Disease;

                _productId = value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }

    public FormProduct(IProductRepository productRepository, IStoreRepository storeRepository)
    {
        InitializeComponent();
        _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
        _storeRepository = storeRepository ?? throw new ArgumentNullException(nameof(storeRepository));

        // Инициализация CheckedListBox значениями перечисления
        foreach (var elem in Enum.GetValues(typeof(ProductTypeEnum)))
        {
            checkedListBoxProductType.Items.Add(elem);
        }

        // Инициализация ComboBox для магазинов
        comboBoxStore.DataSource = _storeRepository.ReadStores().ToList();
        comboBoxStore.DisplayMember = "FullInformation";  // Отображаем полную информацию о магазине
        comboBoxStore.ValueMember = "Id";        // Значение это идентификатор магазина
    }

    private void ButtonSave_Click(object sender, EventArgs e)
    {
        try
        {
            // Проверка на заполненность всех обязательных полей
            if (checkedListBoxProductType.CheckedItems.Count == 0 ||
                string.IsNullOrWhiteSpace(textBoxPower.Text) ||
                string.IsNullOrWhiteSpace(textBoxPrice.Text) ||
                comboBoxStore.SelectedValue == null ||
                string.IsNullOrWhiteSpace(textBoxThickness.Text))
            {
                throw new Exception("Имеются незаполненные поля.");
            }

            // Сохранение данных
            if (_productId.HasValue)
            {
                _productRepository.UpdateProduct(CreateProduct(_productId.Value));
            }
            else
            {
                _productRepository.CreateProduct(CreateProduct(0));
            }

            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonCancel_Click(object sender, EventArgs e) => Close();

    private Product CreateProduct(int id)
    {
        // Сбор значений из CheckedListBox
        ProductTypeEnum productType = ProductTypeEnum.None;
        foreach (var elem in checkedListBoxProductType.CheckedItems)
        {
            productType |= (ProductTypeEnum)elem;
        }

        // Получение идентификатора магазина из ComboBox (это целое число)
        int storeId = (int)comboBoxStore.SelectedValue!;

        // Создание сущности Product
        return Product.CreateEntity(
            id,
            productType,
            double.Parse(textBoxPower.Text),
            int.Parse(textBoxPrice.Text),
            storeId,  // Передаем идентификатор магазина, а не адрес
            float.Parse(textBoxThickness.Text),
            textBoxDisease.Text
        );
    }
}
