﻿using ProjectOpticsStore.Entities;
using ProjectOpticsStore.Repositories;
using System.ComponentModel;
using Unity;

namespace ProjectOpticsStore.Forms;

public partial class FormProducts : Form
{
    private readonly IUnityContainer _container;
    private readonly IProductRepository _productRepository;

    public FormProducts(IUnityContainer container, IProductRepository productRepository)
    {
        InitializeComponent();
        _container = container ?? throw new ArgumentNullException(nameof(container));
        _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
    }
    private void FormProducts_Load(object sender, EventArgs e)
    {
        try
        {
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при загрузке", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        try
        {
            // Открытие формы для добавления нового товара
            _container.Resolve<FormProduct>().ShowDialog();
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при добавлении", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonUpd_Click(object sender, EventArgs e)
    {
        if (!TryGetIdentifierFromSelectedRow(out var productId))
        {
            return;
        }

        try
        {
            // Открытие формы редактирования существующего товара
            var form = _container.Resolve<FormProduct>();
            form.Id = productId;
            form.ShowDialog();
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при изменении", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonDel_Click(object sender, EventArgs e)
    {
        if (!TryGetIdentifierFromSelectedRow(out var productId))
        {
            return;
        }

        if (MessageBox.Show("Удалить запись?", "Удаление", MessageBoxButtons.YesNo) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            // Удаление товара
            _productRepository.DeleteProduct(productId);
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при удалении", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void LoadList()
    {
        // Загружаем список товаров
        var products = _productRepository.ReadProducts().ToList();

        // Преобразуем данные с FullInformation магазинов
        var productsWithStoreInfo = products.Select(product =>
        {
            var store = _container.Resolve<IStoreRepository>().ReadStoreById(product.Store);
            return new
            {
                product.Id,
                product.Type,
                product.Power,
                product.Price,
                Store = store?.FullInformation, //деваться некуда, приходится передавать FullInformation
                product.Thickness,
                product.Disease
            };
        }).ToList();

        // Привязываем данные к DataGridView
        dataGridViewProducts.DataSource = productsWithStoreInfo;

        // Настраиваем заголовки колонок на основе DisplayName
        ApplyDisplayNames();

        dataGridViewProducts.Columns["Id"].Visible = false;
    }

    //После обновления метода LoadList у меня все русские заголовки с DisplayName перестали работать, поэтому
    //мною было принято решение о создании нового метода, который может и костыльно, но это фиксит. Главное что оно работает
    private void ApplyDisplayNames()
    {
        foreach (var column in dataGridViewProducts.Columns.Cast<DataGridViewColumn>())
        {
            // Получаем свойство, соответствующее колонке, из типа Product
            var property = typeof(Product).GetProperty(column.DataPropertyName);
            if (property == null)
            {
                continue;
            }

            // Читаем атрибут DisplayName
            var displayNameAttr = property.GetCustomAttributes(typeof(DisplayNameAttribute), false)
                                           .FirstOrDefault() as DisplayNameAttribute;

            // Если атрибут найден, задаем его значение в качестве заголовка
            if (displayNameAttr != null)
            {
                column.HeaderText = displayNameAttr.DisplayName;
            }
        }
    }


    private bool TryGetIdentifierFromSelectedRow(out int id)
    {
        id = 0;
        if (dataGridViewProducts.SelectedRows.Count < 1)
        {
            MessageBox.Show("Нет выбранной записи", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }

        // Получение идентификатора выбранного товара
        id = Convert.ToInt32(dataGridViewProducts.SelectedRows[0].Cells["Id"].Value);
        return true;
    }
}