﻿using Microsoft.Extensions.Logging;
using ProjectOpticsStore.Repositories;
using ProjectOpticsStore.Entities;
using ProjectOpticsStore.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectOpticsStore.Reports
{
    public class ChartReport
    {
        private readonly ISupplyRepository _supplyRepository; // Репозиторий для поставок
        private readonly IProductRepository _productRepository; // Репозиторий для продуктов
        private readonly ILogger<ChartReport> _logger;

        // Конструктор с зависимостями
        public ChartReport(ISupplyRepository supplyRepository, IProductRepository productRepository, ILogger<ChartReport> logger)
        {
            _supplyRepository = supplyRepository ?? throw new ArgumentNullException(nameof(supplyRepository));
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository)); // Для получения типа продукта
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        // Метод для создания графика
        public bool CreateChart(string filePath, DateTime dateTime)
        {
            try
            {
                // Создание PDF с графиком
                new PdfBuilder(filePath)
                    .AddHeader("Распределение поставок по товарам") // Заголовок
                    .AddPieChart($"Поставки на {dateTime:dd MMMM yyyy}", GetData(dateTime)) // График
                    .Build();
                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        // Метод для получения данных для графика
        private List<(string Caption, double Value)> GetData(DateTime dateTime)
        {
            // Получаем все поставки для заданной даты
            var supplies = _supplyRepository
                .ReadSupplies()
                .Where(x => x.OperationDate.Date == dateTime.Date) // Фильтруем по дате
                .GroupBy(x => x.ProductId) // Группируем по ProductId
                .Select(g => new
                {
                    ProductId = g.Key,
                    TotalQuantity = g.Sum(x => x.Quantity)
                })
                .ToList();

            // Для каждого ProductId находим тип товара через репозиторий продуктов
            return supplies
                .Select(x =>
                {
                    var product = _productRepository.ReadProductById(x.ProductId); // Получаем продукт по ProductId
                    var productType = product.Type.ToString(); // Получаем тип товара (ProductTypeEnum)
                    return (productType, (double)x.TotalQuantity); // Возвращаем тип товара и общее количество
                })
                .ToList();
        }
    }
}
