﻿using DocumentFormat.OpenXml.Wordprocessing;
using Microsoft.Extensions.Logging;
using ProjectOpticsStore.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ProjectOpticsStore.Reports;

public class TableReport
{
    private readonly IOrderRepository _orderRepository;
    private readonly ISupplyRepository _supplyRepository;
    private readonly ILogger<TableReport> _logger;

    // Заголовки таблицы
    // Типо Order - заказ - количество ушло - заказчик, Поставка - количество пришло - магазин.
    // Поэтому в первой строчке Заказчик/Магазин а не только Заказчик. Все согласно Ерке.

    internal static readonly string[] item = { "Заказчик", "Магазин", "Дата", "Количество пришло", "Количество ушло" };

    public TableReport(IOrderRepository orderRepository, ISupplyRepository supplyRepository,
        ILogger<TableReport> logger)
    {
        _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
        _supplyRepository = supplyRepository ?? throw new ArgumentNullException(nameof(supplyRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    // Метод для создания таблицы
    public bool CreateTable(string filePath, int productID, DateTime startDate, DateTime endDate)
    {
        try
        {
            // Получение данных
            var data = GetData(productID, startDate, endDate);

            // Создание Excel документа
            new ExcelBuilder(filePath)
                .AddHeader("Сводка по движению товаров", 0, 4)
                .AddParagraph($"За период c {startDate:dd.MM.yyyy} по {endDate:dd.MM.yyyy}", 0)
                .AddTable(new[] { 15, 15, 12, 15, 15 }, data)  // Ширины столбцов
                .Build();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    // Метод для получения данных
    private List<string[]> GetData(int productId, DateTime startDate, DateTime endDate)
    {
        // Получение данных заказов
        var ordersData = _orderRepository
            .ReadOrders(dateFrom: startDate, dateTo: endDate, productId: productId)
            .Select(order => new
            {
                CustomerName = order.CustomerName,
                StoreName = string.Empty, // Магазин не указывается для заказов
                Date = order.DateCreated,
                CountIn = (int?)null,
                CountOut = (int?)order.OrderProducts.Sum(op => op.Quantity)
            });

        // Получение данных поставок
        var suppliesData = _supplyRepository
            .ReadSupplies(dateFrom: startDate, dateTo: endDate, productId: productId)
            .Select(supply => new
            {
                CustomerName = string.Empty, // Заказчик не указывается для поставок
                StoreName = supply.StoreName,
                Date = supply.OperationDate,
                CountIn = (int?)supply.Quantity,
                CountOut = (int?)null
            });

        // Объединение и сортировка данных
        var combinedData = ordersData
            .Union(suppliesData)
            .OrderBy(x => x.Date);

        // Формирование таблицы
        return new List<string[]> { item } // Добавляем заголовок таблицы
            .Union(
                combinedData.Select(entry => new string[] {entry.CustomerName ?? string.Empty, entry.StoreName ?? string.Empty, entry.Date.ToString("dd.MM.yyyy"), entry.CountIn?.ToString("N0") ?? string.Empty, entry.CountOut?.ToString("N0") ?? string.Empty
                })
            )
            .Union(new[]
            {
                    // Итоговая строка
                    new string[] { "Всего", "", "", combinedData.Sum(entry => entry.CountIn ?? 0).ToString("N0"), combinedData.Sum(entry => entry.CountOut ?? 0).ToString("N0")}
            })
            .ToList();
    }
}
