﻿using Dapper;
using DocumentFormat.OpenXml.Wordprocessing;
using Microsoft.AspNetCore.Http.Extensions;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectOpticsStore.Entities;
using ProjectOpticsStore.Entities.Enums;
using Unity;

namespace ProjectOpticsStore.Repositories.Implementations;

internal class OrderRepository : IOrderRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<OrderRepository> _logger;

    public OrderRepository(IConnectionString connectionString, ILogger<OrderRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateOrder(Order order)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(order));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO Orders (CustomerId, DateCreated, Status)
                VALUES (@CustomerId, @DateCreated, @Status);
                SELECT MAX(Id) FROM Orders";
            var OrderId = connection.QueryFirst<int>(queryInsert, order, transaction);

            var querySubInsert = @"
                INSERT INTO Orders_Products (OrderId, ProductId, Quantity)
                VALUES (@OrderId, @ProductId, @Quantity)";
            foreach (var elem in order.OrderProducts)
            {
                connection.Execute(querySubInsert, new { OrderId, elem.ProductId, elem.Quantity }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public IEnumerable<Order> ReadOrders(DateTime? dateFrom = null, DateTime? dateTo = null, int? customerId = null, int? productId = null)
    {
        _logger.LogInformation("Получение всех объектов");

        try
        {
            var builder = new QueryBuilder();

            if (dateFrom.HasValue)
            {
                builder.AddCondition("o.DateCreated >= @dateFrom");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("o.DateCreated <= @dateTo");
            }
            if (customerId.HasValue)
            {
                builder.AddCondition("o.CustomerId = @customerId");
            }
            if (productId.HasValue)
            {
                builder.AddCondition("op.ProductId = @productId");
            }

            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);

            var querySelect = $@"
            SELECT o.Id, 
                   o.DateCreated, 
                   o.Status, 
                   c.FullName AS CustomerName,
                   os.Name AS StatusName,
                   p.Id AS ProductId,
                   p.Type AS Type,
                   op.Quantity
            FROM Orders o
            LEFT JOIN Customers c ON c.Id = o.CustomerId
            LEFT JOIN OrderStatus os ON os.Id = o.Status
            INNER JOIN Orders_Products op ON op.OrderId = o.Id
            LEFT JOIN Products p ON p.Id = op.ProductId

            {builder.Build()}";

            _logger.LogDebug("Выполняется SQL-запрос: {Query}", querySelect);

            var orderDict = new Dictionary<int, List<Order_Product>>();

            var orders = connection.Query<Order, Order_Product, Order>(querySelect,
            (order, orderProduct) =>
            {
                if (!orderDict.TryGetValue(order.Id, out var products))
                {
                    products = new List<Order_Product>();
                    orderDict.Add(order.Id, products);
                }

                products.Add(Order_Product.CreateElement(order.Id, orderProduct.ProductId, orderProduct.Type, orderProduct.Quantity));
                return order;
            },
            splitOn: "ProductId",
            param: new { dateFrom, dateTo, customerId, productId });

            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(orders));

            return orderDict.Select(x =>
            {
                var order = orders.First(y => y.Id == x.Key);
                order.SetOrderProducts(x.Value);
                return order;
            }).ToArray();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}