﻿using Dapper;
using DocumentFormat.OpenXml.Office2010.Excel;
using DocumentFormat.OpenXml.Presentation;
using DocumentFormat.OpenXml.Wordprocessing;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectOpticsStore.Entities;
using System.Windows.Forms;

namespace ProjectOpticsStore.Repositories.Implementations;

internal class SupplyRepository : ISupplyRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<SupplyRepository> _logger;

    public SupplyRepository(IConnectionString connectionString, ILogger<SupplyRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateSupply(Supply supply)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(supply));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO Supplies (StoreId, OperationDate, ProductId, Quantity)
                VALUES (@StoreId, @OperationDate, @ProductId, @Quantity)";
            connection.Execute(queryInsert, supply);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void DeleteSupply(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Supplies
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<Supply> ReadSupplies(DateTime? dateFrom = null, DateTime? dateTo = null, int? storeId = null, int? productId = null)
    {
        _logger.LogInformation("Получение всех объектов");

        try
        {
            var builder = new QueryBuilder();

            if (dateFrom.HasValue)
            {
                builder.AddCondition("su.OperationDate >= @dateFrom");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("su.OperationDate <= @dateTo");
            }
            if (storeId.HasValue)
            {
                builder.AddCondition("su.StoreId = @storeId");
            }
            if (productId.HasValue)
            {
                builder.AddCondition("su.ProductId = @productId");
            }

            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);

            var querySelect = $@"
            SELECT 
                su.*,
                --p.Type AS ProductName,
                TRIM(
                CASE WHEN p.Type & 1 <> 0 THEN 'Очки, ' ELSE '' END ||
                CASE WHEN p.Type & 2 <> 0 THEN 'Контактные линзы, ' ELSE '' END ||
                CASE WHEN p.Type & 4 <> 0 THEN 'Солнцезащитные очки, ' ELSE '' END ||
                CASE WHEN p.Type & 8 <> 0 THEN 'Оправа, ' ELSE '' END ||
                CASE WHEN p.Type & 16 <> 0 THEN 'Футляры, ' ELSE '' END
            , ', ') AS ProductName,
                CONCAT(s.Address, ' ', s.Organisation) AS StoreName
            FROM Supplies su
            LEFT JOIN Products p ON p.Id = su.ProductId
            LEFT JOIN Stores s ON s.Id = su.StoreId
            {builder.Build()}";

            var supplies = connection.Query<Supply>(querySelect, new { dateFrom, dateTo, storeId, productId });

            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(supplies));

            return supplies;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
